#include "ReleaseConfiguration.hpp"
#include "ReleaseVelocity.hpp"
#include "ReleasePoint.hpp"
#include <cmath>

ReleaseConfiguration::ReleaseConfiguration(double release_alt, state_type wind_vector, double indicated_air_speed, position target_pos)
{
	this->alt = release_alt;
	this->IAS = indicated_air_speed;
	this->wind = wind_vector;
	this->target = target_pos;

	setWind();
	calc_point_of_release( );
}

void ReleaseConfiguration::setWind()
{
	double wind_magnitude = sqrt(this->wind[0]*this->wind[0] + this->wind[1]*this->wind[1] + this->wind[2]*this->wind[2]);

	// If the wind is insignificantly small, set the wind values
	if( wind_magnitude < 1)
	{
		this->wind[0] = 0.01;
		this->wind[1] = 0.0;
		this->wind[2] = 0.0;
	}
}

void ReleaseConfiguration::calc_point_of_release( )
{

	// Calculate release velocity vector
	direction = new ReleaseVelocity( this->wind , this->IAS );
	this->release_vel = direction->calc_release_velocity( );

	// Create a state vector containing the information at a given point of release
	state_type x = {{ this->release_vel[0] , this->release_vel[1] , this->release_vel[2] , this->target[0] , this->target[1] , this->alt }};

	// Calculate the release point given the initial velocity and the desired point of impact
	point = new ReleasePoint( this->wind , x , 0.01 );
	this->release_point = point->calc_release_point( this->alt );

	state_type wind_unit_vector = direction->get_wind_unit_vector( );

	release_point[0] = wind_unit_vector[0];
	release_point[1] = wind_unit_vector[1];
	release_point[2] = wind_unit_vector[2];

	delete direction;
	delete point;

}

state_type ReleaseConfiguration::get_release_point( )
{
	return this->release_point;
}
