#include "ReleaseVelocity.hpp"
#include <cmath>

ReleaseVelocity::ReleaseVelocity( state_type wind_input , int true_air_speed )
{
	this->wind[0] = wind_input[0];
	this->wind[1] = wind_input[1];
	this->wind[2] = wind_input[2];
	this->IAS = true_air_speed;

	calc_wind_unit( );
	create_wind_unit_vector( );
	calc_release_velocity_decomposed( );

	this->release_velocity[0] = 0.0;
	this->release_velocity[1] = 0.0;
	this->release_velocity[2] = 0.0;

}

void ReleaseVelocity::calc_wind_unit( )
{
	this->wind_magnitude = sqrt( (this->wind[0]*this->wind[0]) + (this->wind[1]*this->wind[1]) + (this->wind[2]*this->wind[2]) );
	this->wind_unit_x = this->wind[0]/this->wind_magnitude;
	this->wind_unit_y = this->wind[1]/this->wind_magnitude;
	this->wind_unit_z = this->wind[2]/this->wind_magnitude;
}

void ReleaseVelocity::create_wind_unit_vector( )
{
	this->wind_unit_vector[0] = this->wind_unit_x;
	this->wind_unit_vector[1] = this->wind_unit_y;
	this->wind_unit_vector[2] = this->wind_unit_z;
}

void ReleaseVelocity::calc_release_velocity_decomposed( )
{
	// Indicated Airspeed in direction against the wind + the effect of the wind
	this->release_velocity_x = -1*this->IAS*this->wind_unit_x+this->wind[0];
	this->release_velocity_y = -1*this->IAS*this->wind_unit_y+this->wind[1];
	this->release_velocity_z = -1*this->IAS*this->wind_unit_z+this->wind[2];
}

state_type ReleaseVelocity::calc_release_velocity( )
{
	this->release_velocity[0] = this->release_velocity_x;
	this->release_velocity[1] = this->release_velocity_y;
	//this->release_velocity[2] = this->release_velocity_z;

	return this->release_velocity;
}

state_type ReleaseVelocity::get_wind_unit_vector( )
{
	return this->wind_unit_vector;
}

double ReleaseVelocity::get_wind_magnitude( )
{
	return this->wind_magnitude;
}
