//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Siri Holthe Mathisen                                             *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>
#include <fstream>
#include <iostream>

using namespace std;

namespace PrecisionAirdrop
{
  namespace DropLoad
  {
    using DUNE_NAMESPACES;

    struct Task: public DUNE::Tasks::Task
    {
	
      //! Constructor.
      //! @param[in] name task name.
      //! @param[in] ctx context.
      Task(const std::string& name, Tasks::Context& ctx):
        DUNE::Tasks::Task(name, ctx)
      {
    	  bind<IMC::PayloadRelease>(this);
    	  //Make a streamer
		  ofstream myFile;
		  //Export the right pin: Pin17 on J6
		  myFile.open("/sys/class/gpio/export");
		  if(!myFile.is_open())
		  {
			  inf("Error opening /sys/class/gpio/export");
		  }
		  myFile << "36";
		  myFile.close();
		  inf("Added GPIO36");
		  //Set the direction to out
		  myFile.open("/sys/class/gpio/gpio36/direction");
		  if(!myFile.is_open())
		  {
			  inf("Error opening /sys/class/gpio/gpio36/direction");
		  }
		  myFile << "out";
		  myFile.close();
		  inf("Set direction to out");
		  myFile.open("/sys/class/gpio/gpio36/value");
		  if(!myFile.is_open())
		  {
			  inf("Error opening /sys/class/gpio/gpio36/value");
		  }
		  myFile << "1";
		  myFile.close();
		  inf("Set value to 1");
      }

      //! Update internal state with new parameter values.
      void
      onUpdateParameters(void)
      {
      }

      //! Reserve entity identifiers.
      void
      onEntityReservation(void)
      {
      }

      //! Resolve entity names.
      void
      onEntityResolution(void)
      {
      }

      //! Acquire resources.
      void
      onResourceAcquisition(void)
      {
      }

      //! Initialize resources.
      void
      onResourceInitialization(void)
      {
    	  //Make a streamer
		  ofstream myFile;
		  //Export the right pin: Pin17 on J6
		  myFile.open("/sys/class/gpio/export");
		  if(!myFile.is_open())
		  {
			  inf("Error opening /sys/class/gpio/export");
		  }
		  myFile << "36";
		  myFile.close();
		  inf("Added GPIO36");
		  //Set the direction to out
		  myFile.open("/sys/class/gpio/gpio36/direction");
		  if(!myFile.is_open())
		  {
			  inf("Error opening /sys/class/gpio/gpio36/direction");
		  }
		  myFile << "out";
		  myFile.close();
		  inf("Set direction to out");
		  myFile.open("/sys/class/gpio/gpio36/value");
		  if(!myFile.is_open())
		  {
			  inf("Error opening /sys/class/gpio/gpio36/value");
		  }
		  myFile << "1";
		  myFile.close();
		  inf("Set value to 1");
      }

      //! Release resources.
      void
      onResourceRelease(void)
      {
      }

      void
      consume(const IMC::PayloadRelease* msg)
      {

    	  if(msg->confirm == 1)
    	  {
    		  //Turn off the drop mechanism

    		  //Make a streamer
			  ofstream myFile;
			  myFile.open("/sys/class/gpio/gpio36/value");
			  if(!myFile.is_open())
			  {
				  inf("Error opening /sys/class/gpio/gpio36/value");
			  }
			  myFile << "1";
			  myFile.close();
			  inf("Set value to 1");

    	  }else{
    		  //Turn on the drop mechanism

			  //Make a streamer
			  ofstream myFile;
			  myFile.open("/sys/class/gpio/gpio36/value");
			  if(!myFile.is_open())
			  {
				  inf("Error opening /sys/class/gpio/gpio36/value");
			  }
			  myFile << "0";
			  myFile.close();
			  inf("Set value to 0");
    	  }
      }

      //! Main loop.
      void
      onMain(void)
      {
        while (!stopping())
        {
          waitForMessages(1.0);
        }
      }
    };
  }
}

DUNE_TASK
