//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Siri Holthe Mathisen                                             *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <stdlib.h>
#include <time.h>


using namespace std;

enum DestinationEntity{RELEASE_ON_TARGET = 1, STRAIGHT_LINE, DUBINS_PATH, GROUND_STATION};

struct GeoPosition
{
    fp64_t lat;
    fp64_t lon;
    fp64_t height;
};

namespace PrecisionAirdrop
{
  namespace GroundUnit
  {
    using DUNE_NAMESPACES;

    struct Task: public DUNE::Tasks::Task
    {
    	bool m_confirmed;
    	bool m_release_position_received;
    	bool m_released;
    	double m_tolerance;
    	IMC::Target m_target;
    	IMC::Target m_release_pos;
    	IMC::Target m_actual_release;
    	IMC::TrajectoryPoint m_target_pos_ned;
    	IMC::TrajectoryPoint m_release_pos_ned;
    	IMC::TrajectoryPoint m_actual_release_pos_ned;
    	const IMC::EstimatedState* m_state;
    	char filename[256];
    	ofstream myFile;

      //! Constructor.
      //! @param[in] name task name.
      //! @param[in] ctx context.
      Task(const std::string& name, Tasks::Context& ctx):
        DUNE::Tasks::Task(name, ctx)
      {
    	  //Agdenes-koordinater
    	  //m_target.lat = 63.628806/180*3.1415926535;//0.65719152998;
    	  //m_target.lon = 9.727369/180*3.1415926535;//-2.13544287275;
    	  //HIL-koordinater
    	  m_target.lat = 37.648020/180*3.1415926535;
    	  m_target.lon = -122.363753/180*3.1415926535;
    	  m_target.z = 50;
    	  m_target.z_units = IMC::Z_HEIGHT;
    	  m_target.setDestinationEntity(DUBINS_PATH);

    	  // Set tolerance in meters
    	  m_tolerance = 10.0;
    	  std::ostringstream oss;
    	  oss << m_tolerance;
    	  m_target.label = oss.str();

    	  time_t m_time;
    	  m_time = time(0);
    	  char time[80];
    	  struct tm tstruct;
		  tstruct = *localtime(&m_time);
    	  strcpy(filename, "/home/simen/Dropbox/log/");
    	  strftime(time, sizeof(time), "%Y-%m-%d.%X", &tstruct);
    	  strcat(filename, time);
    	  strcat(filename, ".txt");

    	  m_state = NULL;

    	  m_confirmed = false;
    	  bind<IMC::Target>(this);
    	  bind<IMC::EstimatedState>(this);
    	  bind<IMC::PayloadRelease>(this);


		  myFile.open(filename, std::ios::app);
		  if(!myFile.is_open())
		  {
			  inf("Error opening %s", filename);
		  }
		  myFile << "New test" << "\n";
		  myFile << "hit" << '\t' << "dist" << '\t' << "distx" << '\t' << "disty" << '\t'
				  << "rcog" << '\t' << "arcog" << '\t'
				  << "u" << '\t' << "v" << '\t' << "w" << '\t'
				  << "vx" << '\t' << "vy" << '\t' << "vz" << '\t'
				  << "x" << '\t' << "y" << '\t' << "height" << '\t'
				  << "lat" << '\t' << "lon" << '\t'
				  << "alt" << '\n';
		  myFile << "--------------------------------------------------------------------------------------------------------------" << '\n';
		  myFile.close();
      }

      //! Update internal state with new parameter values.
      void
      onUpdateParameters(void)
      {
      }

      //! Reserve entity identifiers.
      void
      onEntityReservation(void)
      {
      }

      //! Resolve entity names.
      void
      onEntityResolution(void)
      {
      }

      //! Acquire resources.
      void
      onResourceAcquisition(void)
      {
      }

      //! Initialize resources.
      void
      onResourceInitialization(void)
      {
    	  m_release_position_received = false;
    	  m_released = false;
      }

      //! Release resources.
      void
      onResourceRelease(void)
      {
      }

      void
      consume(const IMC::PayloadRelease* msg)
      {
    	  double lat;
		  double lon;
		  Coordinates::toWGS84(*m_state, lat, lon);
		  lat = lat/3.1415926535*180;
		  lon = lon/3.1415926535*180;
		  inf("Estimated state at drop: lat: %f, lon: %f", lat, lon);
      }

      void
      consume(const IMC::Target* msg)
      {
    	  int dest = msg->getDestinationEntity();

    	  if(dest == GROUND_STATION)
    	  {

			  if(msg->label == "confirm")
			  {
				  m_confirmed = 1;
				  inf("Target confirmed received");
			  }
			  else if(msg->label == "release")
			  {
				  inf("Received Release Position");
				  m_release_pos.lat = msg->lat;
				  m_release_pos.lon = msg->lon;
				  m_release_pos.cog = msg->cog/3.1415926535*180;
			  }
			  else if(msg->label == "actual_release")
			  {
				  inf("Successfully released");
				  m_released = true;

				  m_actual_release.lat = msg->lat;
				  m_actual_release.lon = msg->lon;
				  m_actual_release.cog = msg->cog/3.1415926535*180;

				  // Repeat sequence
				  m_confirmed = false;

			  }
    	  }
      }
      void
		consume(const IMC::EstimatedState* msg)
		{
    	  m_state = msg;
    	  //inf("Estimated state: lat: %f, lon:%f, height:%f", msg->lat, msg->lon, msg->height);

    	  GeoPosition ref_pos;
    	  ref_pos.lat = msg->lat;
    	  ref_pos.lon = msg->lon;
    	  ref_pos.height = msg->height;

    	  double u = msg->u;
    	  double v = msg->v;
    	  double w = msg->w;

    	  double vx = msg->vx;
    	  double vy = msg->vy;
    	  double vz = msg->vz;

    	  double x = msg->x;
    	  double y = msg->y;
    	  double z = msg->z;

    	  /*
    	   * Set logging options:
    	   * For ReleaseObject, set release_object true. Otherwise logging is setup for SLA and ADP.
    	   * 		If release_object = true: For ball release condition, set ball true. Otherwise halfplane wil be used.
    	   * 		If release_object = false: For long_test, set true.
    	   */
    	  bool release_object = false;
    	  bool ball = true;
    	  bool long_test = true;

    	  if(release_object)
    	  {
    		  /*
    		   * ReleaseObject is logged in two different ways:
    		   * 	In the first way it logs any point within the ball created by the radius m_tolerance.
    		   * 	In the second way it logs only the point created after passing a defined halfplane,
    		   * 	causing the release mechanism to activate.
    		   */

    		  // Convert target WGS-84 to NED-frame
			  WGS84::displacement(ref_pos.lat, ref_pos.lon, ref_pos.height, m_target.lat, m_target.lon,
							m_target.z, &m_target_pos_ned.x, &m_target_pos_ned.y, &m_target_pos_ned.z);

			  if(ball)
			  {

				  double dist_x = m_target_pos_ned.x - msg->x;
				  double dist_y = m_target_pos_ned.y - msg->y;
				  double dist = sqrt(dist_x*dist_x + dist_y*dist_y);
				  //inf("dist_x: %f, dist_y: %f, dist: %f", dist_x, dist_y, dist);

				  double lat;
				  double lon;
				  float hae;
				  Coordinates::toWGS84(*msg, lat, lon, hae);
				  lat = lat/3.1415926535*180;
				  lon = lon/3.1415926535*180;

				  int hit = 0;

				  if( dist < m_tolerance )
				  {
					  myFile.open(filename, std::ios::app);
					  if(!myFile.is_open())
					  {
						  inf("Error opening %s", filename);
					  }

					  std::setprecision(10);

					  myFile << std::fixed << std::setprecision(10) << hit << '\t'
							  << dist << '\t' << dist_x << '\t' << dist_y << '\t'
							  << m_release_pos.cog << '\t' << m_actual_release.cog << '\t'
							  << u << '\t' << v << '\t' << w << '\t'
							  << vx << '\t' << vy << '\t' << vz << '\t'
							  << x << '\t' << y << '\t'
							  << lat << '\t' << lon << '\t'
							  << msg->alt << '\t' << '\n';
					  myFile.close();
				  }


			  }
			  else
			  {

				  // Convert target WGS-84 to NED-frame
				  WGS84::displacement(ref_pos.lat, ref_pos.lon, ref_pos.height, m_actual_release.lat, m_actual_release.lon,
						  m_actual_release.z, &m_actual_release_pos_ned.x, &m_actual_release_pos_ned.y, &m_actual_release_pos_ned.z);

				  // Check distance between reported release position and desired release position
				  double dist_x = m_actual_release_pos_ned.x - m_target_pos_ned.x;
				  double dist_y = m_actual_release_pos_ned.y - m_target_pos_ned.y;
				  double dist = sqrt(dist_x*dist_x + dist_y*dist_y);
				  //inf("dist_x: %f, dist_y: %f, dist: %f", dist_x, dist_y, dist);

				  double lat;
				  double lon;
				  float hae;
				  Coordinates::toWGS84(*msg, lat, lon, hae);
				  lat = lat/3.1415926535*180;
				  lon = lon/3.1415926535*180;

				  int hit = 0;

				  if(m_released)
				  {
					  myFile.open(filename, std::ios::app);
					  if(!myFile.is_open())
					  {
						  inf("Error opening %s", filename);
					  }

					  std::setprecision(10);

					  myFile << std::setprecision(10) << hit << '\t'
							  << dist << '\t' << dist_x << '\t' << dist_y << '\t'
							  << m_release_pos.cog << '\t' << m_actual_release.cog << '\t'
							  << u << '\t' << v << '\t' << w << '\t'
							  << vx << '\t' << vy << '\t' << vz << '\t'
							  << x << '\t' << y << '\t'
							  << lat << '\t' << lon << '\t'
							  << msg->alt << '\t' << '\n';
					  myFile.close();

					  m_released = false;
				  }


			  }

    	  }
    	  else
    	  {
    		  /*
    		   * SLA and DBA are tested with a long test (only logging point of release) and a short test that logs
    		   * the position continuously in order to observe the path. The latter indicates the point of release
    		   * by setting the hit-variable to 1 in the log-file.
    		   */

    		  if(long_test)
    		  {
    			  // Only extract information upon release

    			  // Convert target WGS-84 to NED-frame
				  WGS84::displacement(ref_pos.lat, ref_pos.lon, ref_pos.height, m_release_pos.lat, m_release_pos.lon,
						  m_release_pos.z, &m_release_pos_ned.x, &m_release_pos_ned.y, &m_release_pos_ned.z);

				  // Convert target WGS-84 to NED-frame
				  WGS84::displacement(ref_pos.lat, ref_pos.lon, ref_pos.height, m_actual_release.lat, m_actual_release.lon,
						  m_actual_release.z, &m_actual_release_pos_ned.x, &m_actual_release_pos_ned.y, &m_actual_release_pos_ned.z);

				  double dist_x = m_actual_release_pos_ned.x - m_release_pos_ned.x;
				  double dist_y = m_actual_release_pos_ned.y - m_release_pos_ned.y;
				  double dist = sqrt(dist_x*dist_x + dist_y*dist_y);
				  //inf("dist_x: %f, dist_y: %f, dist: %f", dist_x, dist_y, dist);

				  double lat;
				  double lon;
				  float hae;
				  Coordinates::toWGS84(*msg, lat, lon, hae);
				  lat = lat/3.1415926535*180;
				  lon = lon/3.1415926535*180;

				  int hit = 0;

				  if(m_released)
				  {
					  myFile.open(filename, std::ios::app);
					  if(!myFile.is_open())
					  {
						  inf("Error opening %s", filename);
					  }

					  std::setprecision(10);

					  myFile << std::setprecision(10) << hit << '\t'
							  << dist << '\t' << dist_x << '\t' << dist_y << '\t'
							  << m_release_pos.cog << '\t' << m_actual_release.cog << '\t'
							  << u << '\t' << v << '\t' << w << '\t'
							  << vx << '\t' << vy << '\t' << vz << '\t'
							  << x << '\t' << y << '\t'
							  << lat << '\t' << lon << '\t'
							  << msg->alt << '\t' << '\n';
					  myFile.close();

					  m_released = false;
				  }

    		  }
    		  else
    		  {

    		  // Convert target WGS-84 to NED-frame
			  WGS84::displacement(ref_pos.lat, ref_pos.lon, ref_pos.height, m_release_pos.lat, m_release_pos.lon,
					  m_release_pos.z, &m_release_pos_ned.x, &m_release_pos_ned.y, &m_release_pos_ned.z);

			  /* Switch between 'm_release_pos_ned' and 'm_target_pos_ned' to choose
			   * whether to track distance between current position and release position or
			   * track distance between current position and target position
			   */
			  double dist_x = m_release_pos_ned.x - msg->x;
			  double dist_y = m_release_pos_ned.y - msg->y;
			  double dist = sqrt(dist_x*dist_x + dist_y*dist_y);
			  inf("dist_x: %f, dist_y: %f, dist: %f", dist_x, dist_y, dist);

			  double lat;
			  double lon;
			  float hae;
			  Coordinates::toWGS84(*msg, lat, lon, hae);
			  lat = lat/3.1415926535*180;
			  lon = lon/3.1415926535*180;

			  int hit = 0;

			  if(m_released)
			  {
				  war("UAV within the given coordinates");
				  inf("Distance: %f, Body-fixed Velocity: %f, %f, %f", dist, u, v, w);
				  hit = 1;
				  m_released = false;
			  }

			  myFile.open(filename, std::ios::app);
			  if(!myFile.is_open())
			  {
				  inf("Error opening %s", filename);
			  }

			  std::setprecision(10);

			  myFile << std::setprecision(10) << hit << '\t'
					  << dist << '\t' << dist_x << '\t' << dist_y << '\t'
					  << m_release_pos.cog << '\t' << m_actual_release.cog << '\t'
					  << u << '\t' << v << '\t' << w << '\t'
					  << vx << '\t' << vy << '\t' << vz << '\t'
					  << x << '\t' << y << '\t'
					  << lat << '\t' << lon << '\t'
					  << msg->alt << '\t' << '\n';
			  myFile.close();
    		  }
    	  }


    	  //double lat;
    	  //double lon;
    	  //Coordinates::toWGS84(*msg, lat, lon);
    	  //lat = lat/3.1415926535*180;
    	  //lon = lon/3.1415926535*180;
    	  //double target_lon = m_target.lon/3.1415926535*180;
    	  //double target_lat = m_target.lat/3.1415926535*180;
    	  //inf("Reference frame: lat: %f, lon: %f", msg->lat, msg->lon);
    	  //inf("Estimated state: lat: %f, lon: %f", lat, lon);
    	  //inf("Target: lat %f, lon: %f", target_lat, target_lon);
		}

      //! Main loop.
      void
      onMain(void)
      {

        while (!stopping())
        {

        	if(m_confirmed == false)
        	{
        		dispatch(m_target);
        		inf("Sending target");
        	}
        	else
        	{
        		//get_input(&m_target);
        	}
          waitForMessages(1.0);
        }
      }
    };
  }
}

DUNE_TASK
