#include "WaypointGenerator.h"
#include <cmath>
#include "DUNE/DUNE.hpp"
#include <iostream>



double const WaypointGenerator::pi = 3.1415;
double const WaypointGenerator::max_bank_angle_deg = 30.0;
double const WaypointGenerator::g = 9.81;
double const WaypointGenerator::shift_const = 4.0;


WaypointGenerator::WaypointGenerator( state_type release_point_input , double true_air_speed_input , waypoint ref_ned_frame )
{
	this->wind_unit_vector[0] = release_point_input[0];
	this->wind_unit_vector[1] = release_point_input[1];
	this->wind_unit_vector[2] = release_point_input[2];

	this->release_point[0] = release_point_input[3];
	this->release_point[1] = release_point_input[4];
	this->release_point[2] = release_point_input[5];



	this->TAS = true_air_speed_input;

	this->ref_lat = ref_ned_frame[0];
	this->ref_lon = ref_ned_frame[1];

	deg2rad( );
	calc_max_turn_radius( );
	generate_pre_release_wp( );
	generate_post_release_wp( );
	generate_wp_list( );
	generate_wp_list_wgs84( );
}

void WaypointGenerator::deg2rad( )
{
	this->max_bank_angle_rad = (this->max_bank_angle_deg/180)*this->pi;
}

void WaypointGenerator::calc_max_turn_radius( )
{

	this->max_turn_radius = (this->TAS*this->TAS)/(this->g*tan(this->max_bank_angle_rad));

}

void WaypointGenerator::generate_pre_release_wp( )
{
	this->pre_release[0] = this->release_point[0]+(this->wind_unit_vector[0]*this->shift_const*this->max_turn_radius);
	this->pre_release[1] = this->release_point[1]+(this->wind_unit_vector[1]*this->shift_const*this->max_turn_radius);
	this->pre_release[2] = this->release_point[2]+(this->wind_unit_vector[2]*this->shift_const*this->max_turn_radius);
}

void WaypointGenerator::generate_post_release_wp( )
{
	this->post_release[0] = this->release_point[0]-(this->wind_unit_vector[0]*this->max_turn_radius);
	this->post_release[1] = this->release_point[1]-(this->wind_unit_vector[1]*this->max_turn_radius);
	this->post_release[2] = this->release_point[2]-(this->wind_unit_vector[2]*this->max_turn_radius);
}

void WaypointGenerator::generate_wp_list( )
{
	this->wpl[0] = this->pre_release;
	this->wpl[1] = this->release_point;
	this->wpl[2] = this->post_release;
}

void WaypointGenerator::generate_wp_list_wgs84( )
{
	// Reset latitude and longitude to NED-frame origin
	this->lat = this->ref_lat;
	this->lon = this->ref_lon;

	// Waypoint 1 - Pre-release
	WGS84::displace( this->pre_release[0] , this->pre_release[1] , &lat , &lon );
	this->wpl_wgs84[0][0] = this->lat;
	this->wpl_wgs84[0][1] = this->lon;
	this->wpl_wgs84[0][2] = this->pre_release[2];

	// Reset latitude and longitude to NED-frame origin
	this->lat = this->ref_lat;
	this->lon = this->ref_lon;

	// Waypoint 2 - Release-point
	WGS84::displace( this->release_point[0] , this->release_point[1] , &lat , &lon );
	this->wpl_wgs84[1][0] = this->lat;
	this->wpl_wgs84[1][1] = this->lon;
	this->wpl_wgs84[1][2] = this->release_point[2];

	// Reset latitude and longitude to NED-frame origin
	this->lat = this->ref_lat;
	this->lon = this->ref_lon;

	// Waypoint 3 - Post-release
	WGS84::displace( this->post_release[0] , this->post_release[1] , &lat , &lon );
	this->wpl_wgs84[2][0] = this->lat;
	this->wpl_wgs84[2][1] = this->lon;
	this->wpl_wgs84[2][2] = this->post_release[2];

}

waypointlist WaypointGenerator::get_wp_list( )
{
	return wpl;
}

waypointlist WaypointGenerator::get_wp_list_wgs84( )
{
	return wpl_wgs84;
}
