//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Simen Fuglaas                                                    *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

namespace PrecisionAirdrop
{
	namespace TestInput
	{
		using DUNE_NAMESPACES;

		struct Task: public DUNE::Tasks::Periodic
		{
			bool target_sent;
			//! Constructor.
			//! @param[in] name task name.
			//! @param[in] ctx context.
			Task(const std::string& name, Tasks::Context& ctx):
			DUNE::Tasks::Periodic(name, ctx)
			{
			}

			//! Reserve entity identifiers.
			void
			onEntityReservation(void)
			{
				inf("Starting: %s", resolveEntity(getEntityId()).c_str());
				inf("Starting: %s", resolveEntity(getEntityId()).c_str());
			}

			//! Initialize resources.
			void
			onResourceInitialization(void)
			{
				target_sent = false;
			}

			void
			task(void)
			{
				IMC::EstimatedState e_state;
				e_state.setSourceEntity(getEntityId());
				e_state.lat = 0.0;
				e_state.lon = 0.0;
				e_state.height = 30.0;
				e_state.alt = 30.0;
				e_state.vx = 28.0;
				e_state.vy = 0.0;
				e_state.vz = 0.0;

				dispatch(e_state);

				if(!target_sent)
				{
					IMC::Target target;
					target.setSourceEntity(getEntityId());
					target.lat = 0.0;
					target.lon = 0.0;
					target.z = 30.0;
					target.z_units = IMC::Z_HEIGHT;

					dispatch(target);
					target_sent = true;
				}

				IMC::EstimatedStreamVelocity stream_vel;
				stream_vel.setSourceEntity(getEntityId());
				stream_vel.x = 0.0;
				stream_vel.y = 0.0;
				stream_vel.z = 0.0;

				dispatch(stream_vel);

				IMC::IndicatedSpeed ind_speed;
				ind_speed.setSourceEntity(getEntityId());
				ind_speed.value = 28.0;

				dispatch(ind_speed);

			}
		};
	}
}

DUNE_TASK
