//***************************************************************************
// Copyright 2007-2014 Universidade do Porto - Faculdade de Engenharia      *
// Laboratório de Sistemas e Tecnologia Subaquática (LSTS)                  *
//***************************************************************************
// This file is part of DUNE: Unified Navigation Environment.               *
//                                                                          *
// Commercial Licence Usage                                                 *
// Licencees holding valid commercial DUNE licences may use this file in    *
// accordance with the commercial licence agreement provided with the       *
// Software or, alternatively, in accordance with the terms contained in a  *
// written agreement between you and Universidade do Porto. For licensing   *
// terms, conditions, and further information contact lsts@fe.up.pt.        *
//                                                                          *
// European Union Public Licence - EUPL v.1.1 Usage                         *
// Alternatively, this file may be used under the terms of the EUPL,        *
// Version 1.1 only (the "Licence"), appearing in the file LICENCE.md       *
// included in the packaging of this file. You may not use this work        *
// except in compliance with the Licence. Unless required by applicable     *
// law or agreed to in writing, software distributed under the Licence is   *
// distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF     *
// ANY KIND, either express or implied. See the Licence for the specific    *
// language governing permissions and limitations at                        *
// https://www.lsts.pt/dune/licence.                                        *
//***************************************************************************
// Author: Simen Fuglaas                                                    *
//***************************************************************************

// DUNE headers.
#include <DUNE/DUNE.hpp>

namespace PrecisionAirdrop
{
	namespace TestOutput
	{
		using DUNE_NAMESPACES;

		struct Task: public DUNE::Tasks::Task
		{
			//! Constructor.
			//! @param[in] name task name.
			//! @param[in] ctx context.
			Task(const std::string& name, Tasks::Context& ctx):
			DUNE::Tasks::Task(name, ctx)
			{
				bind<IMC::DesiredPath>(this);
			}

			void
			consume(const IMC::DesiredPath* path)
			{
				inf("Source (DUNE instance) ID is: %d", path->getSource());
				inf("Source entity (Task instance) ID is: %d", path->getSourceEntity());
				inf("Start coordinate: [%f, %f, %f]", path->start_lat, path->start_lon, path->start_z);
				inf("End coordinate: [%f, %f, %f]", path->end_lat, path->end_lon, path->end_z);
				inf("%d", path->flags);
			}
			//! Main loop.
			void
			onMain(void)
			{
				while (!stopping())
					{
						waitForMessages(1.0);
					}
			}
		};
	}
}

DUNE_TASK
