clear all;
close all;
clc;

%%

% Define constants

% Physical properties of object
A = pi*0.05^2; % Cross-sectional area of a sphere
C_D = 0.47; % Drag constant for a sphere
m = 0.2; % Mass of the object


rho = 1.269; % Air density at +5 degrees Celsius
b = 0.5*A*C_D*rho;
g = 9.81;

iter = 10;

% Wind velocity
w = 5;
wind_angle = 180+45;%pi+pi/4;
v_w = [w*cos(wind_angle);w*sin(wind_angle);0];
wind_uncertainty = 0.5;
wind_angle_uncert = 10;%/180*pi;
wind_magn_range = (w*(1-wind_uncertainty)):...
                   w*wind_uncertainty/iter:(w*(1+wind_uncertainty));
wind_angle_range = (wind_angle-wind_angle_uncert):...
                    (wind_angle_uncert/iter):(wind_angle+wind_angle_uncert);

% wind_angle_range = 0:0.1:4

% norm_angle = normpdf(wind_angle_range,mean(wind_angle_range),0.1745/3);
norm_angle = normpdf(wind_angle_range,wind_angle,1);%std(wind_angle_range)/3);
% norm_angle = normpdf(wind_angle_range,0,1);
norm_magn = normpdf(wind_magn_range,w,1);%std(wind_magn_range)/3);

%norm_angle = norm_angle/(max(norm_angle));

mu = [w wind_angle];
%sigma = cov(norm_magn, norm_angle);%[var(norm_magn) 0; 0 var(norm_angle)];
[X1,X2] = meshgrid(wind_magn_range, wind_angle_range);

norm = mvnpdf([X1(:) X2(:)], mu);%, sigma);
norm = reshape(norm,length(wind_magn_range), length(wind_angle_range));

F_norm_angle = @(wind_angle_range)normpdf(wind_angle_range,wind_angle,std(wind_angle_range)/3);
F_norm_magn = @(wind_magn_range)normpdf(wind_magn_range,w,std(wind_magn_range)/3);
% int_norm_angle = integral(F_norm_angle, wind_angle_range(11), wind_angle_range(12))

figure
plot(wind_angle_range,norm_angle);
title('norm angle')
figure
plot(wind_magn_range,norm_magn);
title('norm magn')
figure
contour(wind_magn_range, wind_angle_range, norm);
view(0,90);
title('norm')
figure
mesh(wind_magn_range, wind_angle_range, norm);
view(0,90);
title('norm')


% Initial velocity
% v_init: Indicated Air Speed
v_init = 28;
wind_angle = wind_angle/180*pi;
v_0 = [v_init*cos(pi/4)+w*cos(wind_angle);v_init*sin(pi/4)+w*sin(wind_angle);0];

% Initial position
x_0_bx = 0;
x_0_by = 0;
x_0_alt = 100;
x_0 = [x_0_bx;x_0_by;x_0_alt];

mu_pos = [2.5 x_0_by]; 

bx_uncert = 5;
by_uncert = 5;
% x_uncert = cos(pi/4)*bx_uncert - sin(pi/4)*by_uncert;
% y_uncert = sin(pi/4)*bx_uncert + cos(pi/4)*by_uncert;
% bx_range = 0:x_uncert/iter/2:x_uncert;
% by_range = -y_uncert:y_uncert/iter:y_uncert;
% [bX1,bX2] = meshgrid(x_range, y_range);
% sigma_pos = [100 0; 0 1];
% norm_pos = mvnpdf([bX1(:) bX2(:)], mu_pos, sigma_pos);
% norm_pos = reshape(norm_pos,length(bx_range), length(by_range));


bx_range = 0:bx_uncert/iter/2:bx_uncert;
by_range = -by_uncert:by_uncert/iter:by_uncert;
[bX1,bX2] = meshgrid(bx_range, by_range);
sigma_pos = [100 0; 0 1];
norm_pos = mvnpdf([bX1(:) bX2(:)], mu_pos, sigma_pos);
norm_pos = reshape(norm_pos,length(bx_range), length(by_range));

pd_x_rel = makedist('Uniform', 'lower', 0, 'upper', bx_uncert);
pdf_x_rel = pdf(pd_x_rel, bx_range);
norm_y_rel = normpdf(by_range);

% norm_dist = cat(3, pdf_x_rel, norm_y_rel);
% norm_dist = reshape(norm_dist,length(bx_range), length(by_range));

figure
plot(bx_range, pdf_x_rel);
title('norm x rel pos')
figure
plot(by_range,norm_y_rel);
title('norm y rel pos')
figure
surf(bx_range, by_range, norm_pos);
view(0,90);
title('norm pos')
% figure
% surf(bx_range, by_range, norm_dist);
% view(0,90);
% title('norm pos')

% Simulation time
dt = 0.01;
tspan = 0:dt:6;

% Integration time
tend = length(tspan);

% Preallocating the position vector
x = zeros(tend,3);

% Initiate impact variable
impact = false;

% Initialize ground level matrix 100x100 m
ground30m = zeros(1000,1000);
ground50m = zeros(1000,1000);
ground100m = zeros(1000,1000);

ground30m_initpos = zeros(1000,1000);
ground50m_initpos = zeros(1000,1000);
ground100m_initpos = zeros(1000,1000);

% Initialize bool variables checking for impact at varies altitudes
impact30m = false;
impact50m = false;
impact100m = false;



% x_dist_x = [-rel_radius:0.1:rel_radius];
% x_dist_y = [-rel_radius:0.1:rel_radius];
% [X1,X2] = meshgrid(x_dist_x,x_dist_y);
% F_pos = mvnpdf([X1(:) X2(:)],mu_x,1);
% contour(F_pos)

% Iterations
tot_iter = length(wind_magn_range)*length(wind_angle_range)+length(by_range)*length(bx_range);
iter_count = 1;

%% Simulation

% Create waitbar to show progress
h = waitbar(0,'Please wait...');



% Simulate position given varying wind
for i=1:(length(wind_magn_range))
    for j=1:(length(wind_angle_range))

        
        w = wind_magn_range(i);
        wind_angle = wind_angle_range(j);
        wind_angle = wind_angle/180*pi;
        v_w = [w*cos(wind_angle);w*sin(wind_angle);0];

        % Solve differential system
        [t,v] = ode45(@(t,v) freeFallFunc(t,v,v_w,b,m,g), tspan, v_0);

        % Calculate position over time
        for ts=1:tend

            if ts == 1
                x(ts,1) = x_0(1);
                x(ts,2) = x_0(2);
                x(ts,3) = x_0(3);
            elseif ~impact100m %if ~(impact30m && impact50m && impact100m)
                x(ts,1) = x(ts-1,1) + v(ts-1,1)*dt;
                x(ts,2) = x(ts-1,2) + v(ts-1,2)*dt;
                x(ts,3) = x(ts-1,3) + v(ts-1,3)*dt;

                if x(ts,3) < 70 && ~impact30m
                    % 30 m
                    impact30m = true;
                    x_impact30m = x(ts,:);
                elseif x(ts,3) < 50 && ~impact50m
                    % 50 m
                    impact50m = true;
                    x_impact50m = x(ts,:);
                elseif x(ts,3) < 0 && ~impact100m
                    % 100 m
                    impact100m = true;
                    x_impact100m = x(ts,:);
                end
            else
                break;
            end
        end
        % Increase resolution to decimeter
        x_incr_reso_30m = x_impact30m*10;
        x_incr_reso_50m = x_impact50m*10;
        x_incr_reso_100m = x_impact100m*10;

        % Round decimal to nearest integer
        x_rounded_30m = round(x_incr_reso_30m(:));
        x_rounded_50m = round(x_incr_reso_50m(:));
        x_rounded_100m = round(x_incr_reso_100m(:));

        % Update ground matrices
        ground30m(x_rounded_30m(1),x_rounded_30m(2)) =+ norm(i,j);
        ground50m(x_rounded_50m(1),x_rounded_50m(2)) =+ norm(i,j);
        ground100m(x_rounded_100m(1),x_rounded_100m(2)) =+ norm(i,j);

        % Reset impact variables
        impact30m = false;
        impact50m = false;
        impact100m = false;

        % Update waitbar
        iter_count = iter_count+1;
        waitbar(iter_count/tot_iter,h);

    end
end

% Simulate position given varying initial position
for m=1:(length(bx_range))
    for n=1:(length(by_range))

        x_rel = [cos(pi/4)*bx_range(m) - sin(pi/4)*by_range(n);...
                sin(pi/4)*bx_range(m) + cos(pi/4)*by_range(n); x_0_alt];
        delta_x = [x_rel(1) - x_0(1); x_rel(2) - x_0(2)];
        
%         figure(98)
%         plot(x_rel(1), x_rel(2))
%         hold on


        
        % Increase resolution to decimeter
        delta_x = delta_x*10;
        
        % Round decimal to nearest integer
        delta_x = round(delta_x(:));
        
        % Rotate 90 degrees for usage with circshift()
        %delta_x = [0 -1; 1 0]*delta_x;
        
        % Shift ground matrix
        temp_ground30m_initpos = circshift(ground30m, delta_x);
        temp_ground50m_initpos = circshift(ground50m, delta_x);
        temp_ground100m_initpos = circshift(ground100m, delta_x);
        
        % Multiply by the probability distribution
        temp_ground30m_initpos = temp_ground30m_initpos*norm_pos(n,m);
        temp_ground50m_initpos = temp_ground50m_initpos*norm_pos(n,m);
        temp_ground100m_initpos = temp_ground100m_initpos*norm_pos(n,m);
        
        % Update ground matrix
        ground30m_initpos = ground30m_initpos + temp_ground30m_initpos;
        ground50m_initpos = ground50m_initpos + temp_ground50m_initpos;
        ground100m_initpos = ground100m_initpos + temp_ground100m_initpos;

        % Update waitbar
        iter_count = iter_count+1;
        waitbar(iter_count/tot_iter,h);
        
        
% x_start = 25;
% y_start = 25;
% x_end = 45;
% y_end = 45;
% n_points = 200;
%         figure(97)
%         surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),temp_ground100m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
%         view(0,90);
        
%         figure(99)
%         plot(delta_x(1), delta_x(2))
%         hold on

    end
end

close(h);

%% Plots

x_start = 25;
y_start = 25;
x_end = 45;
y_end = 45;
n_points = 200;

% figure
% contour(linspace(30,50,200),linspace(30,50,200),ground30m(300:500-1,300:500-1));

% include wind
figure
surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);
figure
mesh(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);

figure
surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);
figure
mesh(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);

figure
surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);
figure
mesh(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);

% Include wind and pos
figure
surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);
figure
mesh(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);

figure
surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);
figure
mesh(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);

figure
surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);
figure
mesh(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
view(0,90);

% Calculate actual drop position
% x(:,1) = x(:,1) - x(t_impact,1);
% x(:,2) = x(:,2) - x(t_impact,2);

% Plots
% figure(1)
% plot(t,v(:,1))
% hold on;
% figure(2)
% plot(t,v(:,2))
% hold on;
% figure(3)
% plot(t,v(:,3))
% hold on;
% figure(4)
% plot3(x(:,1), x(:,2),x(:,3))
% grid on;
% hold on;