clear all;
close all;
clc;

%%

% Define constants

% Physical properties of object
A = pi*0.05^2; % Cross-sectional area of a sphere
C_D = 0.47; % Drag constant for a sphere
m = 0.2; % Mass of the object


rho = 1.269; % Air density at +5 degrees Celsius
b = 0.5*A*C_D*rho;
g = 9.81;

iter = 500;

% Wind velocity
w = 5;
wind_angle = pi+pi/4;%180+45;%
v_w = [w*cos(wind_angle);w*sin(wind_angle);0];
wind_uncertainty = 0.5;
wind_angle_uncert = pi/2;%180/180*pi;
wind_magn_range = (w*(1-wind_uncertainty)):...
                   w*wind_uncertainty/iter:(w*(1+wind_uncertainty));
wind_angle_range = (wind_angle-wind_angle_uncert):...
                    (wind_angle_uncert/iter):(wind_angle+wind_angle_uncert);

% wind_angle_range = 0:0.1:4

% norm_angle = normpdf(wind_angle_range,mean(wind_angle_range),0.1745/3);
norm_angle = normpdf(wind_angle_range,wind_angle,1);%std(wind_angle_range)/3);
% norm_angle = normpdf(wind_angle_range,0,1);
norm_magn = normpdf(wind_magn_range,w,1);%std(wind_magn_range)/3);

%norm_angle = norm_angle/(max(norm_angle));

mu = [w wind_angle];
%sigma = cov(norm_magn, norm_angle);%[var(norm_magn) 0; 0 var(norm_angle)];
[X1,X2] = meshgrid(wind_magn_range, wind_angle_range);

sigma = [std(wind_magn_range)/3 0; 0 std(wind_angle_range)/3];

norm = mvnpdf([X1(:) X2(:)], mu, sigma);
norm = reshape(norm,length(wind_magn_range), length(wind_angle_range));

F_norm_angle = @(wind_angle_range)normpdf(wind_angle_range,wind_angle,std(wind_angle_range)/3);
F_norm_magn = @(wind_magn_range)normpdf(wind_magn_range,w,std(wind_magn_range)/3);
% int_norm_angle = integral(F_norm_angle, wind_angle_range(11), wind_angle_range(12))

% figure
% plot(wind_angle_range,norm_angle);
% title('norm angle')
% figure
% plot(wind_magn_range,norm_magn);
% title('norm magn')
% figure
% contour(wind_magn_range, wind_angle_range, norm);
% view(0,90);
% title('norm')
figure
h1 = surf(wind_magn_range, wind_angle_range, norm);
set(h1,'LineStyle','none');
view(0,90);
title('Multivariate Normal Distribution of the wind')
xlabel('Wind Magnitude [m/s]')
ylabel('Wind Angle (rad)')
colorbar;



% Initial velocity
% v_init: Indicated Air Speed
v_init = 28;
%wind_angle = wind_angle/180*pi;
v_0 = [v_init*cos(pi/4)+w*cos(wind_angle);v_init*sin(pi/4)+w*sin(wind_angle);0];

% Initial position
x_0_bx = 0;
x_0_by = 0;
x_0_alt = 100;
x_0 = [x_0_bx;x_0_by;x_0_alt];

% Simulation time
dt = 0.01;
tspan = 0:dt:6;

% Integration time
tend = length(tspan);

% Preallocating the position vector
x = zeros(tend,3);

% Initiate impact variable
impact = false;

% Initialize ground level matrix 100x100 m
ground30m = zeros(1000,1000);
ground50m = zeros(1000,1000);
ground100m = zeros(1000,1000);

ground30m_initpos = zeros(1000,1000);
ground50m_initpos = zeros(1000,1000);
ground100m_initpos = zeros(1000,1000);

% Initialize bool variables checking for impact at varies altitudes
impact30m = false;
impact50m = false;
impact100m = false;

% Expected impact position
poi30 = [];
poi50 = [];
poi100 = [];

% Extract HIL data
    format long;

    % Set path
    path(path, 'C:\Users\simenfu\Dropbox\Master Thesis\Matlab\Log\Dubins Path')
    path(path, 'C:\Users\simenfu\Dropbox\Master Thesis\Matlab\')

    % Extract first file
    filename = '2014-05-19.10_22_56_deviation_.txt';
    data_1 = extractDist( filename );

    % Extract second file
    filename = '2014-05-19.17_29_51_deviation_.txt';
    data_2 = extractDist( filename );

    % Merge data
    data = [data_1; data_2];

    % Create sample vector
    n = 1:length(data);

    direction = zeros(length(data),1);
    speed = zeros(length(data),1);

    for i = n
        speed(i) = sqrt(data(i,5)^2 + data(i,6)^2 + data(i,7)^2);
        direction(i) = atan(data(i,6)/data(i,5))-atan(data(i,9)/data(i,8));
        direction(i) = direction(i);%/pi*180;
    end

% Iterations
tot_iter = length(wind_magn_range)*length(wind_angle_range)+length(data);
iter_count = 1;

%% Simulation

% Create waitbar to show progress
path(path, 'C:\Users\simenfu\Dropbox\Master Thesis\Matlab\')
h = waitbar(0,'Please wait...');



% Simulate position given varying wind
for i=1:(length(wind_magn_range))
    for j=1:(length(wind_angle_range))

        
        w_iter = wind_magn_range(i);
        wind_angle_iter = wind_angle_range(j);
        %wind_angle = wind_angle/180*pi;
        v_w = [w_iter*cos(wind_angle_iter);w_iter*sin(wind_angle_iter);0];

        % Solve differential system
        [t,v] = ode45(@(t,v) freeFallFunc(t,v,v_w,b,m,g), tspan, v_0);

        % Calculate position over time
        for ts=1:tend

            if ts == 1
                x(ts,1) = x_0(1);
                x(ts,2) = x_0(2);
                x(ts,3) = x_0(3);
            elseif ~impact100m %if ~(impact30m && impact50m && impact100m)
                x(ts,1) = x(ts-1,1) + v(ts-1,1)*dt;
                x(ts,2) = x(ts-1,2) + v(ts-1,2)*dt;
                x(ts,3) = x(ts-1,3) + v(ts-1,3)*dt;

                if x(ts,3) < 70 && ~impact30m
                    % 30 m
                    impact30m = true;
                    x_impact30m = x(ts,:);
                elseif x(ts,3) < 50 && ~impact50m
                    % 50 m
                    impact50m = true;
                    x_impact50m = x(ts,:);
                elseif x(ts,3) < 0 && ~impact100m
                    % 100 m
                    impact100m = true;
                    x_impact100m = x(ts,:);
                end
            else
                break;
            end
        end
        % Increase resolution to decimeter
        x_incr_reso_30m = x_impact30m*10;
        x_incr_reso_50m = x_impact50m*10;
        x_incr_reso_100m = x_impact100m*10;

        % Round decimal to nearest integer
        x_rounded_30m = round(x_incr_reso_30m(:));
        x_rounded_50m = round(x_incr_reso_50m(:));
        x_rounded_100m = round(x_incr_reso_100m(:));

        % Update ground matrices
        ground30m(x_rounded_30m(1),x_rounded_30m(2)) =+ norm(i,j);
        ground50m(x_rounded_50m(1),x_rounded_50m(2)) =+ norm(i,j);
        ground100m(x_rounded_100m(1),x_rounded_100m(2)) =+ norm(i,j);

        % Extract expected Point of Impact
        if w_iter == w && wind_angle_iter == wind_angle
            
            poi30 = x_impact30m;
            poi50 = x_impact50m;
            poi100 = x_impact100m;
            
        end
        
        % Reset impact variables
        impact30m = false;
        impact50m = false;
        impact100m = false;

        % Update waitbar
        iter_count = iter_count+1;
        waitbar(iter_count/tot_iter,h);

    end
end

length_poi30 = sqrt(poi30(1)^2 + poi30(2)^2);
length_poi50 = sqrt(poi50(1)^2 + poi50(2)^2);
length_poi100 = sqrt(poi100(1)^2 + poi100(2)^2);

for m=1:length(data)
    
    % Correct for release position
    x_rel = [cos(pi/4)*data(m,1) - sin(pi/4)*data(m,2);...
            sin(pi/4)*data(m,1) + cos(pi/4)*data(m,2); x_0_alt];
    delta_x = [x_rel(1) - x_0(1); x_rel(2) - x_0(2)];
    
    % Correct for release angle
    rot = [cos(direction(m))*poi30(1) - sin(direction(m))*poi30(2);...
            sin(direction(m))*poi30(1) + cos(direction(m))*poi30(2)];
    delta_x_30 = [rot(1) - poi30(1);rot(2) - poi30(2)];
    delta_x_30 = delta_x + delta_x_30;
    
    rot = [cos(direction(m))*poi50(1) - sin(direction(m))*poi50(2);...
            sin(direction(m))*poi50(1) + cos(direction(m))*poi50(2)];
    delta_x_50 = [rot(1) - poi50(1);rot(2) - poi50(2)];
    delta_x_50 = delta_x + delta_x_50;
    
    rot = [cos(direction(m))*poi100(1) - sin(direction(m))*poi100(2);...
            sin(direction(m))*poi100(1) + cos(direction(m))*poi100(2)];
    delta_x_100 = [rot(1) - poi100(1);rot(2) - poi100(2)];
    delta_x_100 = delta_x + delta_x_100;
    
    
%     dy = length_poi30*sin(direction(m));
%     dX = length_poi30*cos(direction(m));
%     dx = length_poi30-dX;
%     delta_x_30 = [x_rel(1) - x_0(1) - dx; x_rel(2) - x_0(2) - dy];
%     
%     dy = length_poi50*sin(direction(m));
%     dX = length_poi50*cos(direction(m));
%     dx = length_poi50-dX;
%     delta_x_50 = [x_rel(1) - x_0(1) - dx; x_rel(2) - x_0(2) - dy];
%     
%     dy = length_poi100*sin(direction(m));
%     dX = length_poi100*cos(direction(m));
%     dx = length_poi100-dX;
%     delta_x_100 = [x_rel(1) - x_0(1) - dx; x_rel(2) - x_0(2) - dy];
    
    % Increase resolution to decimeter
    delta_x = delta_x*10;
    delta_x_30 = delta_x_30*10;
    delta_x_50 = delta_x_50*10;
    delta_x_100 = delta_x_100*10;
    

    % Round decimal to nearest integer
    delta_x = round(delta_x(:));
    delta_x_30 = round(delta_x_30(:));
    delta_x_50 = round(delta_x_50(:));
    delta_x_100 = round(delta_x_100(:));
    
    % Shift ground matrix
    temp_ground30m_initpos = circshift(ground30m, delta_x_30);
    temp_ground50m_initpos = circshift(ground50m, delta_x_50);
    temp_ground100m_initpos = circshift(ground100m, delta_x_100);

    % Find probability
    p_pos = 1/length(data);
    
    % Multiply by the probability distribution
    temp_ground30m_initpos = temp_ground30m_initpos*p_pos;
    temp_ground50m_initpos = temp_ground50m_initpos*p_pos;
    temp_ground100m_initpos = temp_ground100m_initpos*p_pos;

    % Update ground matrix
    ground30m_initpos = ground30m_initpos + temp_ground30m_initpos;
    ground50m_initpos = ground50m_initpos + temp_ground50m_initpos;
    ground100m_initpos = ground100m_initpos + temp_ground100m_initpos;

    % Update waitbar
    iter_count = iter_count+1;
    waitbar(iter_count/tot_iter,h);
    
end

close(h);

%% Plots

x_start = 10;
y_start = 10;
x_end = 60;
y_end = 60;
n_points = 500;

% figure
% contour(linspace(30,50,200),linspace(30,50,200),ground30m(300:500-1,300:500-1));

% include wind
figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Probability Distribution of Free Fall from 30m')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_30.eps');

figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Probability Distribution of Free Fall from 50m')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_50.eps');

figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Probability Distribution of Free Fall from 100m')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_100.eps');

% Include wind and pos
figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Probability Distribution of Free Fall from 30m with HIL simulated initial value')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_HIL_30.eps');

figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Probability Distribution of Free Fall from 50m with HIL simulated initial value')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_HIL_50.eps');


figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m_initpos(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Probability Distribution of Free Fall from 100m with HIL simulated initial value')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_HIL_100.eps');
%%
figure
    h1 = surf(wind_magn_range, wind_angle_range, norm);
    set(h1,'LineStyle','none');
    view(0,90);
    title('Multivariate Normal Distribution of the Wind')
    xlabel('Wind Magnitude [m/s]')
    ylabel('Wind Angle (rad)')
    colorbar;
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind.eps');
    
%% Difference

ground30m_difference = ground30m - ground30m_initpos;
figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground30m_difference(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Difference between ideal and HIL simulated probability distributions')
    xlabel('x [m]')
    ylabel('y [m]')
    % Save as .eps
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_HIL_30_diff.eps');


ground50m_difference = ground50m - ground50m_initpos;
figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground50m_difference(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Difference between ideal and HIL simulated probability distributions')
    xlabel('x [m]')
    ylabel('y [m]')
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_HIL_50_diff.eps');

ground100m_difference = ground100m - ground100m_initpos;
figure
    h1 = surf(linspace(x_start,x_end,n_points),linspace(y_start,y_end,n_points),ground100m_difference(x_start*10:x_end*10-1,y_start*10:y_end*10-1));
    view(0,90);
    set(h1,'LineStyle','none');
    colorbar;
    title('Difference between ideal and HIL simulated probability distributions')
    xlabel('x [m]')
    ylabel('y [m]')
    set(gcf,'color','w');
    export_fig ('C:\Users\simenfu\Dropbox\Master Thesis\Thesis\PDF_wind_HIL_100_diff.eps');
