%clear;

BEGIN_ACADO;                                % Always start with "BEGIN_ACADO". 
    
    acadoSet('problemname', 'trajectory');     % Set your problemname. If you 
                                                    % skip this, all files will
                                                    % be named "myAcadoProblem"
    
    
    DifferentialState       x1 x2 x3 x4;     % The differential states
    Control                 u1 u2;           % The controls
    %Disturbance             w;              % The disturbances
    %Parameter               T;            % The free parameters
    
    
    init = [0.0, 10.0, 10.0,22.0,0.0];
    pinit = [0.0, 0.0];
    
    %% Differential Equation
    %f = acado.DifferentialEquation(0,T);       % Set the differential equation object
    f = acado.DifferentialEquation();       % Set the differential equation object
    
    f.add(dot(x1) == x3*cos(x4));           % Speed x-direction
    f.add(dot(x2) == x3*sin(x4));           % Speed y-direction
    f.add(dot(x3) == u1);                   % Acceleration [m/s*s]
    f.add(dot(x4) == u2);                   % Headingrate [rad/s]
    
                                            % Write down your ODE. You can add 
                                            % multiple ODE's with f.add() when you 
                                            % have multiple states. To print an ODE 
                                            % to the screen, use
                                            % f.differentialList{1}.toString

    %% Optimal Control Problem
    %ocp = acado.OCP(0.0, T);          % Set up the Optimal Control Problem (OCP)
                                            % Start at 0s, control in 20
                                            % intervals upto 20s
    ocp = acado.OCP(0.0, 50.0, 50);          % Set up the Optimal Control Problem (OCP)
                                            % Start at 0s, control in 20
                                            % intervals upto 20s
    h = {x1,x2,x3,x4};
    r = [0.0, 0.0, 22.0, 0.0];
    S = [1 0 0 0;
        0 1 0 0;
        0 0 1 0;
        0 0 0 10];

%     x_w = [1;1;0];
% 
%     min_term = cross(x_w, [x1;x2;0]);
%     
%     h = min_term(3);
%     r = 0.0;
    ocp.minimizeLSQEndTerm(S, h, r);
    %ocp.minimizeMayerTerm(min_term(3));   % Minimize a Mayer term
    %ocp.minimizeMayerTerm(T);
    
    
    ocp.subjectTo( f );                     % Your OCP is always subject to your 
                                            % differential equation
                                            
    ocp.subjectTo( 'AT_START', x1 == init(2) );  % Initial condition
    ocp.subjectTo( 'AT_START', x2 == init(3) );  % Initial condition
    ocp.subjectTo( 'AT_START', x3 == init(4) );  % Initial condition
    ocp.subjectTo( 'AT_START', x4 == init(5) );  % Initial condition
%     ocp.subjectTo( 'AT_END',   x1 == 0.0 );  % End condition
%     ocp.subjectTo( 'AT_END',   x2 == 0.0 );  % End condition
%     ocp.subjectTo( 'AT_END',   x3 == 22.0 );  % End condition
%     ocp.subjectTo( 'AT_END',   x4 == 0.0 );  % End condition
    
    %ocp.subjectTo(  0 <= x4 );      % Bounds
    ocp.subjectTo(  u1 == 0 );
    ocp.subjectTo(  -0.25 <= u2 <= 0.25 );
    %ocp.subjectTo( -0.1 <= w <= 2.1 );
    
    
    %% Optimization Algorithm
    algo = acado.OptimizationAlgorithm(ocp); % Set up the optimization algorithm
    
    
    
%     algo.initializeDifferentialStates(init);
%     algo.initializeParameters(pinit);
    algo.set('INTEGRATOR_TOLERANCE', 1e-5 ); % Set some parameters for the algorithm
    
    
END_ACADO;           % Always end with "END_ACADO".
                     % This will generate a file problemname_ACADO.m. 
                     % Run this file to get your results. You can
                     % run the file problemname_ACADO.m as many
                     % times as you want without having to compile again.



out = trajectory_RUN();                % Run the test. The name of the RUN file
                                            % is problemname_RUN, so in
                                            % this case getting_started_RUN
                                            
%draw;

subplot(3,2,1)
plot(out.STATES(:,2), out.STATES(:,3), 'r')
title('x');

subplot(3,2,2)
plot(out.STATES(:,1), out.STATES(:,5), 'r')
title('x4');


subplot(3,2,3)
plot(out.CONTROLS(:,1), out.CONTROLS(:,3), 'r')
title('u2');

% subplot(2,2,3)
% plot(out.STATES(:,1), out.PARAMETERS(:,2), 'r')
% title('p');

subplot(3,2,4)
plot(out.CONTROLS(:,1), out.CONTROLS(:,2), 'r')
title('u1');

subplot(3,2,5)
plot(out.STATES(:,1), out.STATES(:,4), 'r')
title('x4');