clear;

BEGIN_ACADO;                                % Always start with "BEGIN_ACADO". 
    
    acadoSet('problemname', 'trajectoryMPC');     % Set your problemname. If you 
                                                   % skip this, all files will
                                                   % be named "myAcadoProblem"
                                                   
    acadoSet('results_to_file', false);      % Write results only to workspace
                                             % not to file.
    
    
    DifferentialState       x1 x2 x3 x4;     % The differential states
    Control                 u1 u2;           % The controls
    %Disturbance             w;              % The disturbances
    Parameter               T;            % The free parameters
    
    %% Differential Equation
     f = acado.DifferentialEquation(0,T);       % Set the differential equation object
    
    f.add(dot(x1) == x3*cos(x4));           % Speed x-direction
    f.add(dot(x2) == x3*sin(x4));           % Speed y-direction
    f.add(dot(x3) == u1);                   % Acceleration [m/s*s]
    f.add(dot(x4) == u2);                   % Headingrate [rad/s]
    
                                            % Write down your ODE. You can add 
                                            % multiple ODE's with f.add() when you 
                                            % have multiple states. To print an ODE 
                                            % to the screen, use
                                            % f.differentialList{1}.toString
    
    
    %% Optimal Control Problem
    ocp = acado.OCP(0.0, T);          % Set up the Optimal Control Problem (OCP)
                                            % Start at 0s, control in 20
                                            % intervals upto 20s
    h = [x1,x2,x3,x4];
    r = [0.0, 0.0, 22.0, 0.0];
    S = [1 0 0 0;
        0 1 0 0;
        0 0 1 0;
        0 0 0 1];
    ocp.minimizeLSQEndTerm(S, h, r);
    
    h={xB, xW, vB, vW};                     % the LSQ-Function

    Q = eye(4);                             % The weighting matrix
    Q(1,1) = 10;
    Q(2,2) = 10;
    
    r = zeros(1,4);                         % The reference
        
    ocp.minimizeLSQ( Q, h, r );             % Minimize this Least Squares Term
    %ocp.minimizeLSQ( h, r );               % (Other possibility)
    %ocp.minimizeLSQ( h );                  % (Other possibility)
    
    ocp.subjectTo( f );                     % Your OCP is always subject to your 
                                            % differential equation

    ocp.subjectTo(  -500.0 <= F <= 500.0 ); % Bounds
    ocp.subjectTo(  R == 0.0 ); 

   
    
    %% SETTING UP THE (SIMULATED) PROCESS
    identity = acado.OutputFcn();
    dynamicSystem = acado.DynamicSystem(f, identity);    % Set up a dynamic system with
                                                         % the differential equation and an
                                                         % output function
                                                         
    process = acado.Process(dynamicSystem, 'INT_RK45');  % Simulates the process to be controlled 
                                                         % based on a dynamic model.
                                                         % The class Process is one of the two main 
                                                         % building-blocks within the SimulationEnvironment 
                                                         % and complements the Controller. It simulates the 
                                                         % process to be controlled based on a dynamic model.
                                                         
    disturbance = [                                      % The process disturbance matrix consists of one
        0.0       0.00                                   % column with timepoints and one for each disturbance.
        0.85      0.00
        0.9       0.01
        0.95      0.01
        1.0       0.00
        5.0       0.00];
    process.setProcessDisturbance(disturbance);         % Set the process disturbances
    
    
    %% SETTING UP THE MPC CONTROLLER:
    algo = acado.RealTimeAlgorithm(ocp, 0.05);          % The class RealTimeAlgorithm serves as a user-interface 
                                                        % to formulate and solve model predictive control problems.
                                                        
    %algo.set('MAX_NUM_ITERATIONS', 2 );                 % Set some algorithm parameters
    
    zeroReference = acado.StaticReferenceTrajectory();  % Allows to define a static reference trajectory that 
                                                        % the ControlLaw aims to track. 
                                                        
    controller = acado.Controller( algo,zeroReference ); % The controller complements a Process. 
                                                         % It contains an online control law for
                                                         % obtaining the control inputs of a process
    
    
    %% SETTING UP THE SIMULATION ENVIRONMENT,  RUN THE EXAMPLE..
    sim = acado.SimulationEnvironment( 0.0,3.0,process,controller ); % Setup the closed-loop simulations of dynamic systems. 
                                                                     % Simulate from 0 to 3 sec
    
    r = zeros(1,4);                                     % Initilize the states
    sim.init( r );
    

    
END_ACADO;           % Always end with "END_ACADO".
                     % This will generate a file problemname_ACADO.m. 
                     % Run this file to get your results. You can
                     % run the file problemname_ACADO.m as many
                     % times as you want without having to compile again.
                     
    
% Run the test
out = active_damping_RUN();

draw;
